import { __classPrivateFieldGet } from 'tslib';
import * as i0 from '@angular/core';
import { ChangeDetectorRef, Directive } from '@angular/core';
import { isTruthy, wrapMethod } from '@s-libs/js-core';
import { mixInSubscriptionManager, delayOnMicrotaskQueue } from '@s-libs/rxjs-core';
import { Subject, BehaviorSubject, merge, of } from 'rxjs';
import { filter, distinctUntilChanged, map } from 'rxjs/operators';
import { NG_VALUE_ACCESSOR, FormControl } from '@angular/forms';
import { noop } from '@s-libs/micro-dash';

/**
 * Mixes in {@link InjectableSuperclass} as an additional superclass.
 *
 * ```ts
 * class MySubclass extends mixInInjectableSuperclass(MyOtherSuperclass) {
 *   subscribeWithAutoUnsubscribe(observable: Observable<any>) {
 *     this.subscribeTo(observable);
 *   }
 * }
 * ```
 */
function mixInInjectableSuperclass(Base) {
    var _destructionSubject, _a;
    return _a = class extends mixInSubscriptionManager(Base) {
            constructor() {
                super(...arguments);
                _destructionSubject.set(this, new Subject());
                /**
                 * An observable that emits once when this object is destroyed, then completes.
                 */
                this.destruction$ = __classPrivateFieldGet(this, _destructionSubject).asObservable();
            }
            ngOnDestroy() {
                this.unsubscribe();
                __classPrivateFieldGet(this, _destructionSubject).next();
                __classPrivateFieldGet(this, _destructionSubject).complete();
            }
        },
        _destructionSubject = new WeakMap(),
        _a;
}
/**
 * Use as the superclass for anything managed by angular's dependency injection for care-free use of `subscribeTo()`. It simply calls `unsubscribe()` during `ngOnDestroy()`. If you override `ngOnDestroy()` in your subclass, be sure to invoke the super implementation.
 *
 * ```ts
 * @Injectable() // or @Component(), @Directive() or @Pipe(), but consider DirectiveSuperclass
 * class MyThing extends InjectableSuperclass {
 *   constructor(somethingObservable: Observable) {
 *     super();
 *     this.subscribeTo(somethingObservable);
 *   }
 *
 *   ngOnDestroy() {
 *     // if you override ngOnDestroy, be sure to call this too
 *     super.ngOnDestroy();
 *   }
 * }
 * ```
 */
class InjectableSuperclass extends mixInInjectableSuperclass(Object) {
}

var _onChangesRan$;
/**
 * Extend this when creating a directive (including a component, which is a kind of directive) to gain access to the helpers demonstrated below. **Warning:** You _must_ include a constructor in your subclass.
 *
 * ```ts
 * @Component({
 *   selector: "s-color-text",
 *   template: `
 *     <span [style.background]="color">{{ color }}</span>
 *   `,
 *   // note that `bindToInstance()` works even with OnPush change detection
 *   changeDetection: ChangeDetectionStrategy.OnPush,
 * })
 *  class ColorTextComponent extends DirectiveSuperclass {
 *   @Input() prefix?: string;
 *   @Input() prefix2?: string;
 *   color!: string;
 *
 *   constructor(
 *     @Inject("color$") color$: Observable<string>,
 *     injector: Injector,
 *   ) {
 *     super(injector);
 *
 *     // combine everything to calculate `color` and keep it up to date
 *     this.bindToInstance(
 *       "color",
 *       combineLatest(
 *         this.getInput$("prefix"),
 *         this.getInput$("prefix2"),
 *         color$,
 *       ).pipe(map((parts) => parts.filter((p) => p).join(""))),
 *     );
 *   }
 * }
 * ```
 */
// maybe this won't need the fake selector after https://github.com/angular/angular/issues/36427
// eslint-disable-next-line @angular-eslint/directive-class-suffix
class DirectiveSuperclass extends InjectableSuperclass {
    constructor(injector) {
        super();
        /**
         *  Emits the set of `@Input()` property names that change during each call to `ngOnChanges()`.
         */
        this.inputChanges$ = new Subject();
        _onChangesRan$.set(this, new BehaviorSubject(false));
        this.changeDetectorRef = injector.get(ChangeDetectorRef);
    }
    ngOnChanges(changes) {
        this.inputChanges$.next(new Set(Object.getOwnPropertyNames(changes)));
        __classPrivateFieldGet(this, _onChangesRan$).next(true);
    }
    /**
     * @return an observable of the values for one of this directive's `@Input()` properties
     */
    getInput$(key) {
        // Should emit:
        //   - immediately if ngOnChanges was already called
        //   - on the first call to ngOnChanges
        //   - after a delay if ngOnChanges is never called (when nothing is bound to the directive)
        //   - when the value actually changes
        return merge(__classPrivateFieldGet(this, _onChangesRan$).pipe(filter(isTruthy), distinctUntilChanged()), this.inputChanges$, of(0).pipe(delayOnMicrotaskQueue())).pipe(map(() => this[key]), distinctUntilChanged());
    }
    /**
     * Binds an observable to one of this directive's instance variables. When the observable emits the instance variable will be updated, and change detection will be triggered to propagate any changes. Use this an an alternative to repeating `| async` multiple times in your template.
     */
    bindToInstance(key, value$) {
        this.subscribeTo(value$, (value) => {
            this[key] = value;
            this.changeDetectorRef.markForCheck();
        });
    }
}
_onChangesRan$ = new WeakMap();
DirectiveSuperclass.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.0", ngImport: i0, type: DirectiveSuperclass, deps: [{ token: i0.Injector }], target: i0.ɵɵFactoryTarget.Directive });
DirectiveSuperclass.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.0.0", type: DirectiveSuperclass, selector: "[sDirectiveSuperclass]", usesInheritance: true, usesOnChanges: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.0", ngImport: i0, type: DirectiveSuperclass, decorators: [{
            type: Directive,
            args: [{ selector: '[sDirectiveSuperclass]' }]
        }], ctorParameters: function () { return [{ type: i0.Injector }]; } });

/**
 * Use in the `providers` of a component that implements `ControlValueAccessor` to reduce some boilerplate.
 *
 * ```ts
 * @Component({ providers: [provideValueAccessor(MyFormControl)] }
 * class MyFormControl extends BaseFormControl {
 *   // ...
 * }
 * ```
 */
function provideValueAccessor(type) {
    return {
        provide: NG_VALUE_ACCESSOR,
        useExisting: type,
        multi: true,
    };
}
/**
 * Extend this when creating a form control to reduce some boilerplate.
 *
 * This example allows 2-way binding to a number via `[(ngModel)]`, `[formControl]`, or any other technique that leverages the `ControlValueAccessor` interface.
 * ```ts
 * @Component({
 *   template: `
 *     <button (click)="increment()" [disabled]="isDisabled">{{ counter }}</button>
 *   `,
 *   providers: [provideValueAccessor(CounterComponent)],
 * })
 * class CounterComponent extends FormControlSuperclass<number> {
 *   counter = 0;
 *
 *   handleIncomingValue(value: number) {
 *     this.counter = value;
 *   }
 *
 *   increment() {
 *     this.emitOutgoingValue(++this.counter);
 *     this.onTouched();
 *   }
 * }
 * ```
 */
class FormControlSuperclass extends DirectiveSuperclass {
    constructor() {
        super(...arguments);
        /** Call this to emit a new value when it changes. */
        this.emitOutgoingValue = noop;
        /** Call this to "commit" a change, traditionally done e.g. on blur. */
        this.onTouched = noop;
        /** You can bind to this in your template as needed. */
        this.isDisabled = false;
    }
    /** Called as angular propagates value changes to this `ControlValueAccessor`. You normally do not need to use it. */
    writeValue(value) {
        this.handleIncomingValue(value);
        this.changeDetectorRef.markForCheck();
    }
    /** Called as angular sets up the binding to this `ControlValueAccessor`. You normally do not need to use it. */
    registerOnChange(fn) {
        this.emitOutgoingValue = fn;
    }
    /** Called as angular sets up the binding to this `ControlValueAccessor`. You normally do not need to use it. */
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    /** Called as angular propagates disabled changes to this `ControlValueAccessor`. You normally do not need to use it. */
    setDisabledState(isDisabled) {
        this.isDisabled = isDisabled;
        this.changeDetectorRef.markForCheck();
    }
}

/**
 * Extend this when creating a form control that simply wraps an existing form control, to reduce a lot of boilerplate. **Warning:** You _must_ include a constructor in your subclass.
 *
 * Example when you don't need to modify the wrapped control's value:
 * ```ts
 * @Component({
 *   template: `<input [formControl]="formControl">`,
 *   providers: [provideValueAccessor(StringComponent)],
 * })
 * class StringComponent extends WrappedFormControlSuperclass<string> {
 *   // This looks unnecessary, but is required for Angular to provide `Injector`
 *   constructor(injector: Injector) {
 *     super(injector);
 *   }
 * }
 * ```
 *
 * Example when you need to modify the wrapped control's value:
 * ```ts
 * @Component({
 *   template: `<input type="datetime-local" [formControl]="formControl">`,
 *   providers: [provideValueAccessor(DateComponent)],
 * })
 * class DateComponent extends WrappedFormControlSuperclass<Date, string> {
 *   // This looks unnecessary, but is required for Angular to provide `Injector`
 *   constructor(injector: Injector) {
 *     super(injector);
 *   }
 *
 *   protected innerToOuter(inner: string): Date {
 *     return new Date(inner + "Z");
 *   }
 *
 *   protected outerToInner(outer: Date): string {
 *     if (outer === null) {
 *       return ""; // happens during initialization
 *     }
 *     return outer.toISOString().substr(0, 16);
 *   }
 * }
 * ```
 */
class WrappedFormControlSuperclass extends FormControlSuperclass {
    constructor(injector) {
        super(injector);
        /** Bind this to your inner form control to make all the magic happen. */
        this.formControl = new FormControl();
        this.incomingValues$ = new Subject();
        this.subscribeTo(this.setUpOuterToInner$(this.incomingValues$), (inner) => {
            this.formControl.setValue(inner, { emitEvent: false });
        });
        this.subscribeTo(this.setUpInnerToOuter$(this.formControl.valueChanges), (outer) => {
            this.emitOutgoingValue(outer);
        });
        wrapMethod(this.formControl, 'markAsTouched', {
            after: () => {
                this.onTouched();
            },
        });
    }
    /** Called as angular propagates values changes to this `ControlValueAccessor`. You normally do not need to use it. */
    handleIncomingValue(outer) {
        this.incomingValues$.next(outer);
    }
    /** Called as angular propagates disabled changes to this `ControlValueAccessor`. You normally do not need to use it. */
    setDisabledState(isDisabled) {
        if (isDisabled) {
            this.formControl.disable({ emitEvent: false });
        }
        else {
            this.formControl.enable({ emitEvent: false });
        }
        super.setDisabledState(this.isDisabled);
    }
    /**
     * Override this to modify a value coming from the outside to the format needed within this component.
     *
     * For more complex needs, see {@link #setUpOuterToInner$} instead.
     */
    outerToInner(outer) {
        return outer;
    }
    /**
     * Override this to for full control over the stream of values passed in to your subclass.
     *
     * In this example, incoming values are debounced before being passed through to the inner form control
     * ```ts
     * setUpOuterToInner$(outer$: Observable<OuterType>): Observable<InnerType> {
     *   return values$.pipe(
     *     debounce(300),
     *     map((outer) => doExpensiveTransformToInnerValue(outer)),
     *   );
     * }
     * ```
     *
     * For a simple transformation, see {@link #outerToInner} instead.
     */
    setUpOuterToInner$(outer$) {
        return outer$.pipe(map((outer) => this.outerToInner(outer)));
    }
    /**
     * Override this to modify a value coming from within this component to the format expected on the outside.
     *
     * For more complex needs, see {@link #setUpInnerToOuter$} instead.
     */
    innerToOuter(inner) {
        return inner;
    }
    /**
     * Override this to for full control over the stream of values emitted from your subclass.
     *
     * In this example, illegal values are not emitted
     * ```ts
     * setUpInnerToOuter$(inner$: Observable<InnerType>): Observable<OuterType> {
     *   return values$.pipe(
     *     debounce(300),
     *     filter((inner) => isLegalValue(outer)),
     *   );
     * }
     * ```
     *
     * For a simple transformation, see {@link #innerToOuter} instead.
     */
    setUpInnerToOuter$(inner$) {
        return inner$.pipe(map((inner) => this.innerToOuter(inner)));
    }
}

/*
 * Public API Surface of ng-core
 */

/**
 * Generated bundle index. Do not edit.
 */

export { DirectiveSuperclass, FormControlSuperclass, InjectableSuperclass, WrappedFormControlSuperclass, mixInInjectableSuperclass, provideValueAccessor };
//# sourceMappingURL=ng-core.js.map
